#import Pkg

#typeof(Base.find_package("EngeePhased"))    == String || Pkg.develop(path="$(@__DIR__)/EngeePhased")
#typeof(Base.find_package("PlotlyJS"))       == String || Pkg.add("PlotlyJS")

mutable struct Param
    propSpeed :: Int64 
    fc :: Int64                           
    pulseBw :: Float64 
    prf :: Float64 
    fs :: Float64
    txGain :: Float64
    peakPower :: Float64
    matchingCoeff :: Array{Float64}
    metersPerSample :: Float64
    rangeOffset :: Float64
    rangeLoss :: Array{Float64}
    referenceLoss :: Float64
    target1Rcs :: Float64
    target1Pos :: Array{Float64}
    target1Vel :: Array{Float64}
    radar_pos :: Array{Float64}
    radar_vel :: Array{Float64}
    gain :: Int64 
    lossFactor :: Float64 
    referenceTemperature :: Float64
    maxRange :: Float64
    numPulseInt :: Int64
    rangeGates :: Array{Float64}
end

function  initRadarParam()
           
    propSpeed, 
    fc, 
    pulseBw, 
    prf, 
    fs, 
    txGain, 
    peakPower, 
    matchingCoeff, 
    metersPerSample, 
    rangeOffset, 
    rangeLoss, 
    referenceLoss, 
    target1Rcs, 
    target1Pos, 
    target1Vel,
    radar_pos,
    radar_vel,
    gain,
    lossFactor,
    referenceTemperature,
    maxRange,
    numPulseInt,
    rangeGates = calcParams()

                                            
    paramRadar = Param(propSpeed,
                        fc,
                        pulseBw,
                        prf,
                        fs,
                        txGain,
                        peakPower,
                        matchingCoeff,
                        metersPerSample,
                        rangeOffset,
                        rangeLoss,
                        referenceLoss,
                        target1Rcs,
                        target1Pos,
                        target1Vel,
                        radar_pos,
                        radar_vel,
                        gain,
                        lossFactor,
                        referenceTemperature,
                        maxRange,
                        numPulseInt,
                        rangeGates)


    return paramRadar

end

function calcParams()

    # Environment
    propSpeed = 299792458               # Propagation speed
    fc = 1e10                           # Operating frequency
    lambda = propSpeed/fc

    # Constraints
    
    maxRange = 5000                     # Maximum unambiguous range
    rangeRes = 50                       # Required range resolution
    pd = 0.9                            # Probability of detection
    pfa = 1e-6                          # Probability of false alarm
    tgtRcs = 1                          # Required target radar cross section
    numPulseInt = 10                    # Integrate 10 pulses at a time
    
   # Waveform parameters
    
    pulseBw = propSpeed/(2*rangeRes)    # Pulse bandwidth
    pulseWidth = 1/pulseBw              # Pulse width
    prf = propSpeed/(2*maxRange)        # Pulse repetition frequency
    fs = 2*pulseBw

    # Transmitter parameters

    snrMin = albersheim(pd, pfa, numPulseInt)      
    txGain = 20
    peakPower = ((4*pi)^3*noisepow(1/pulseWidth)*maxRange^4* db2pow(snrMin))/(db2pow(2*txGain)*tgtRcs*lambda^2)

    peakPower = 5.2265e3

    # Matched filter parameters

    hwav = EngeePhased.RectangularWaveform(   
        PulseWidth = 1/pulseBw,
        PRF = prf,
        SampleRate = fs
    )
    matchingCoeff = getMatchedFilter(hwav)


    # Delay introduced due to filter

    matchingDelay = size(matchingCoeff,1)-1;
    
    # Time varying gain parameters 
    fastTimeGrid = Vector(0:1/fs:(1/prf-1/fs))
    rangeGates = fastTimeGrid.*propSpeed/2; 
    metersPerSample = rangeGates[2];
    rangeOffset = -rangeGates[2]*matchingDelay;
    
    
    rangeLoss = 2FreeSpacePathLoss(rangeGates,lambda);     
    referenceLoss = 2FreeSpacePathLoss(maxRange,lambda);
    rangeLoss = 2*20log10.(4*pi*rangeGates./lambda).*Integer.(rangeGates./lambda .!= 0)
    referenceLoss =  2*20log10.(4*pi*maxRange./lambda)


    gain = 20
    lossFactor = 0
    referenceTemperature = 290

    # Radar parameters
    
    target1Rcs = 0.6
    target1Pos = [1.9887e3; 0; 0]
    target1Vel = [0; 0; 0]

    # Additional params

    radar_pos = [0; 0; 0]
    radar_vel = [0; 0; 0]
    gain = 20
    lossFactor = 0
    referenceTemperature = 290  
    
    return  propSpeed,
            fc,
            pulseBw,
            prf,
            fs,
            txGain,
            peakPower,
            matchingCoeff,
            metersPerSample,
            rangeOffset,
            rangeLoss,
            referenceLoss,
            target1Rcs,
            target1Pos,
            target1Vel,
            radar_pos,
            radar_vel,
            gain,
            lossFactor,
            referenceTemperature,
            maxRange,
            numPulseInt,
            rangeGates

end


function albersheim(pd, pfa, N)
    A = log(0.62/pfa)
    B = log(pd/(1-pd))
    ABfactor = (A + 0.12*A*B + 1.7*B)
    N1 = 1/sqrt(N)
    N2 = (6.2+4.54/sqrt(N+0.44))/10
    snr = N1*ABfactor^N2

    snr = (10 .*log10(snr)+300)-300

end

function FreeSpacePathLoss(R, lambda)
    L = 4π*R / lambda

    for it ∈ L
        if it < 1
            it = 1
        end
    end
    return L = 10log10.(L)
end

function systemp(nf :: Number,reftemp :: Number = 290)
    nf >= 0 || error("Invalid input")
    nfLinear = db2pow(nf); 
    stemp = reftemp*(nfLinear); # Kelvin
    return stemp
end

function noisepow(nbw :: Number, nf :: Number = 0, reftemp :: Number = 290)
    B = 1.3806504e-23
    npow =  B * systemp(nf,reftemp) * nbw
    return npow 
end
    